#include "GeoMesh.h"

using namespace Geometry;
using namespace std;

//----------------------------------------------------------------------------
Mesh::Mesh():
	mVertexBuffer(0), mSubMeshCount(0), mSubMesh(0), mType(GEO_TRIANGLE_LIST)
{
}

Mesh::~Mesh()
{
	delete[] mSubMesh;
	delete mVertexBuffer;
}


Mesh::Mesh(const Mesh &objmesh) 
{
	// constructor copia
	mVertexBuffer=new VertexBuffer();
	mType=objmesh.mType;
	mSubMeshCount=objmesh.mSubMeshCount;
	mSubMesh = new Geometry::SubMesh[objmesh.mSubMeshCount];
	for(size_t i=0;i<objmesh.mSubMeshCount;i++)
	{
		mSubMesh[i].mSharedVertexBuffer=false;
		mSubMesh[i].mVertexBuffer = new Geometry::VertexBuffer();
		mSubMesh[i].mVertexBuffer->mPosition = new Geometry::Vector3[objmesh.mSubMesh[i].mVertexBuffer->mVertexCount];
		mSubMesh[i].mVertexBuffer->mNormal = new Geometry::Vector3[objmesh.mSubMesh[i].mVertexBuffer->mVertexCount];
		mSubMesh[i].mVertexBuffer->mTexCoords = new Geometry::Vector2[objmesh.mSubMesh[i].mVertexBuffer->mVertexCount];
		mSubMesh[i].mVertexBuffer->mVertexCount = objmesh.mSubMesh[i].mVertexBuffer->mVertexCount;
		mSubMesh[i].mVertexBuffer->mVertexInfo = objmesh.mSubMesh[i].mVertexBuffer->mVertexInfo;
		mSubMesh[i].mType	=	objmesh.mSubMesh[i].mType;
		
		// De momento con esto se parte de un geomesh con vrtices compartidos y se obtiene un geomesh sin vrtices
		// compartidos. Los vrtices se replican en cada uno de los submeshes.
		// HAY QUE CAMBIARLO.
		if(objmesh.mSubMesh[i].mSharedVertexBuffer == true) 
		{
			for(size_t s=0;s<objmesh.mSubMesh[i].mVertexBuffer->mVertexCount;s++)
			{
				//Copiamos mPosition
				if (objmesh.mSubMesh[i].mVertexBuffer->mVertexInfo & Geometry::VERTEX_POSITION)
				{
					mSubMesh[i].mVertexBuffer->mPosition[s].x = objmesh.mVertexBuffer->mPosition[s].x;
					mSubMesh[i].mVertexBuffer->mPosition[s].y = objmesh.mVertexBuffer->mPosition[s].y;
					mSubMesh[i].mVertexBuffer->mPosition[s].z = objmesh.mVertexBuffer->mPosition[s].z;
				}
				//Copiamos mNormal
				if (objmesh.mSubMesh[i].mVertexBuffer->mVertexInfo & Geometry::VERTEX_NORMAL)
				{
					mSubMesh[i].mVertexBuffer->mNormal[s].x = objmesh.mVertexBuffer->mNormal[s].x;
					mSubMesh[i].mVertexBuffer->mNormal[s].y = objmesh.mVertexBuffer->mNormal[s].y;
					mSubMesh[i].mVertexBuffer->mNormal[s].z = objmesh.mVertexBuffer->mNormal[s].z;
				}

				//Copiamos mTexCoords
				if (objmesh.mSubMesh[i].mVertexBuffer->mVertexInfo & Geometry::VERTEX_TEXCOORDS)
				{
					mSubMesh[i].mVertexBuffer->mTexCoords[s].x = objmesh.mVertexBuffer->mTexCoords[s].x;
					mSubMesh[i].mVertexBuffer->mTexCoords[s].y = objmesh.mVertexBuffer->mTexCoords[s].y;
				}
			}
			objmesh.mSubMesh[i].mSharedVertexBuffer = false;
		}
		else
		{
			for(size_t s=0;s<objmesh.mSubMesh[i].mVertexBuffer->mVertexCount;s++)
			{
				//Copiamos mPosition
				if (objmesh.mSubMesh[i].mVertexBuffer->mVertexInfo & Geometry::VERTEX_POSITION)
				{
					mSubMesh[i].mVertexBuffer->mPosition[s].x = objmesh.mSubMesh[i].mVertexBuffer->mPosition[s].x;
					mSubMesh[i].mVertexBuffer->mPosition[s].y = objmesh.mSubMesh[i].mVertexBuffer->mPosition[s].y;
					mSubMesh[i].mVertexBuffer->mPosition[s].z = objmesh.mSubMesh[i].mVertexBuffer->mPosition[s].z;
				}
				//Copiamos mNormal
				if (objmesh.mSubMesh[i].mVertexBuffer->mVertexInfo & Geometry::VERTEX_NORMAL)
				{
					mSubMesh[i].mVertexBuffer->mNormal[s].x = objmesh.mSubMesh[i].mVertexBuffer->mNormal[s].x;
					mSubMesh[i].mVertexBuffer->mNormal[s].y = objmesh.mSubMesh[i].mVertexBuffer->mNormal[s].y;
					mSubMesh[i].mVertexBuffer->mNormal[s].z = objmesh.mSubMesh[i].mVertexBuffer->mNormal[s].z;
				}

				//Copiamos mTexCoords
				if (objmesh.mSubMesh[i].mVertexBuffer->mVertexInfo & Geometry::VERTEX_TEXCOORDS)
				{
					mSubMesh[i].mVertexBuffer->mTexCoords[s].x = objmesh.mSubMesh[i].mVertexBuffer->mTexCoords[s].x;
					mSubMesh[i].mVertexBuffer->mTexCoords[s].y = objmesh.mSubMesh[i].mVertexBuffer->mTexCoords[s].y;
				}
			}

		}
		// Copiar los indices
		mSubMesh[i].mIndexCount=objmesh.mSubMesh[i].mIndexCount;
		mSubMesh[i].mIndex = new Index[mSubMesh[i].mIndexCount];
		memcpy(mSubMesh[i].mIndex,objmesh.mSubMesh[i].mIndex,objmesh.mSubMesh[i].mIndexCount*sizeof(Geometry::Index));

		// Copiar las tiras
		// TENER EN CUENTA SI ES UN MESH DE TIRAS O LISTA DE TRIANGULOS
		__w64 int Desp=0; // desplazamiento entre posiciones de memoria.
		mSubMesh[i].mStripCount=objmesh.mSubMesh[i].mStripCount;
		if (objmesh.mSubMesh[i].mStripCount>0) 
		{
			mSubMesh[i].mStrip = new Index*[objmesh.mSubMesh[i].mStripCount];

			Desp= &(mSubMesh[i].mIndex[0]) - &(objmesh.mSubMesh[i].mIndex[0]);
			for (size_t j=0;j<objmesh.mSubMesh[i].mStripCount;j++) 
			{
				mSubMesh[i].mStrip[j]=objmesh.mSubMesh[i].mStrip[j] + Desp;
			}
		}
	}
}

Mesh &Mesh::operator =(const Geometry::Mesh &objmesh)
{
	// Operador de asignacin
	bool copiados	=	false; // indica si los vrtices compartidos han sido copiados
	mVertexBuffer	=	new VertexBuffer();
	mType					=	objmesh.mType;
	mSubMeshCount	=	objmesh.mSubMeshCount;
	mSubMesh			=	new Geometry::SubMesh[objmesh.mSubMeshCount];

	for(size_t i = 0; i < objmesh.mSubMeshCount; i++)
	{
		//	Debug.
		/*
		cout	<<	"VertexCount["
					<<	i
					<<	"]: "
					<<	objmesh.mSubMesh[i].mVertexBuffer->mVertexCount
					<<	endl;
		*/
		//--------------------------------------------------------------------------------------------

		mSubMesh[i].mSharedVertexBuffer					=	objmesh.mSubMesh[i].mSharedVertexBuffer; //.false;
		mSubMesh[i].mVertexBuffer								=	new Geometry::VertexBuffer();
		mSubMesh[i].mVertexBuffer->mPosition		=	new Geometry::Vector3[objmesh.mSubMesh[i].mVertexBuffer->mVertexCount];
		mSubMesh[i].mVertexBuffer->mNormal			=	new Geometry::Vector3[objmesh.mSubMesh[i].mVertexBuffer->mVertexCount];
		mSubMesh[i].mVertexBuffer->mTexCoords		=	new Geometry::Vector2[objmesh.mSubMesh[i].mVertexBuffer->mVertexCount];
		mSubMesh[i].mVertexBuffer->mVertexCount	=	objmesh.mSubMesh[i].mVertexBuffer->mVertexCount;
		mSubMesh[i].mVertexBuffer->mVertexInfo	=	objmesh.mSubMesh[i].mVertexBuffer->mVertexInfo;
		mSubMesh[i].mType	=	objmesh.mSubMesh[i].mType;

		if (objmesh.mSubMesh[i].mSharedVertexBuffer && !copiados)
		{
			mVertexBuffer	=	mSubMesh[i].mVertexBuffer;
			copiados			=	true;
		}

		for(size_t s = 0; s < objmesh.mSubMesh[i].mVertexBuffer->mVertexCount; s++)
		{
			//Copiamos mPosition
			//if (objmesh.mSubMesh[i].mVertexBuffer->mVertexInfo & Geometry::VERTEX_POSITION)
			//{
				mSubMesh[i].mVertexBuffer->mPosition[s].x = objmesh.mSubMesh[i].mVertexBuffer->mPosition[s].x;
				mSubMesh[i].mVertexBuffer->mPosition[s].y = objmesh.mSubMesh[i].mVertexBuffer->mPosition[s].y;
				mSubMesh[i].mVertexBuffer->mPosition[s].z = objmesh.mSubMesh[i].mVertexBuffer->mPosition[s].z;
			//}
			//Copiamos mNormal
			if (objmesh.mSubMesh[i].mVertexBuffer->mVertexInfo & Geometry::VERTEX_NORMAL)
			{
				mSubMesh[i].mVertexBuffer->mNormal[s].x = objmesh.mSubMesh[i].mVertexBuffer->mNormal[s].x;
				mSubMesh[i].mVertexBuffer->mNormal[s].y = objmesh.mSubMesh[i].mVertexBuffer->mNormal[s].y;
				mSubMesh[i].mVertexBuffer->mNormal[s].z = objmesh.mSubMesh[i].mVertexBuffer->mNormal[s].z;
			}

			//Copiamos mTexCoords
			if (objmesh.mSubMesh[i].mVertexBuffer->mVertexInfo & Geometry::VERTEX_TEXCOORDS)
			{
				mSubMesh[i].mVertexBuffer->mTexCoords[s].x = objmesh.mSubMesh[i].mVertexBuffer->mTexCoords[s].x;
				mSubMesh[i].mVertexBuffer->mTexCoords[s].y = objmesh.mSubMesh[i].mVertexBuffer->mTexCoords[s].y;
			}
		}

		// Copiar los indices
		mSubMesh[i].mIndexCount	=	objmesh.mSubMesh[i].mIndexCount;
		mSubMesh[i].mIndex			=	new Index[mSubMesh[i].mIndexCount];

		memcpy(mSubMesh[i].mIndex,objmesh.mSubMesh[i].mIndex,objmesh.mSubMesh[i].mIndexCount*sizeof(Geometry::Index));

		// Copiar las tiras
		__w64 Desp							=	0; // desplazamiento entre posiciones de memoria.
		mSubMesh[i].mStripCount	=	objmesh.mSubMesh[i].mStripCount;

		if (objmesh.mSubMesh[i].mStripCount>0) 
		{
			mSubMesh[i].mStrip	=	new Index*[objmesh.mSubMesh[i].mStripCount];
			Desp								=	&(mSubMesh[i].mIndex[0]) - &(objmesh.mSubMesh[i].mIndex[0]);
			
			for (size_t j = 0; j < objmesh.mSubMesh[i].mStripCount; j++)
			{
				mSubMesh[i].mStrip[j]	=	objmesh.mSubMesh[i].mStrip[j] + Desp;
			}
		}
		
		//OSCAR
		//Copio los bones
		if (!objmesh.mSubMesh[i].mBones.empty())
		{
			for (int j = 0; j < objmesh.mSubMesh[i].mBones.size(); j++)
			{
				mSubMesh[i].mBones.push_back(objmesh.mSubMesh[i].mBones[j]);
			}
		}
	}

	//OSCAR
	//Copio el nombre del esqueleto
	if (objmesh.hasSkeleton)
	{
		hasSkeleton	=	true;
		
		strcpy(mSkeletonName,objmesh.mSkeletonName);
	}

	//OSCAR
	//Copio los bones
	if (!objmesh.mBones.empty())
	{
		for (int j = 0; j < objmesh.mBones.size(); j++)
		{
			mBones.push_back(objmesh.mBones[j]);
		}
	}

	return	*this;
}

void Mesh::Load(Serializer &s)
{
	bool sharedVertexBuffer	= false;

	//Clear Data
	delete[] mSubMesh;  mSubMesh = 0;
	delete   mVertexBuffer; mVertexBuffer = 0;
	mSubMeshCount = 0;
	
	s.ReadArray(&sharedVertexBuffer,1);
	if (sharedVertexBuffer)
	{
		mVertexBuffer = new VertexBuffer;
		mVertexBuffer->Load(s);
	}
	s.ReadArray(&mSubMeshCount,1);
	mSubMesh = new SubMesh[mSubMeshCount];
	for(size_t i = 0; i < mSubMeshCount; ++i)
	{
		mSubMesh[i].Load(s);
		if (mSubMesh[i].mSharedVertexBuffer && sharedVertexBuffer)
		{
			mSubMesh[i].mVertexBuffer = mVertexBuffer;
		}
	}
}

void Mesh::Save(Serializer &s)
{
	bool sharedVertexBuffer = (mVertexBuffer != 0);
	s.WriteArray(&sharedVertexBuffer, 1);
	if (sharedVertexBuffer)
	{
		mVertexBuffer->Save(s);
	}
	s.WriteArray(&mSubMeshCount,1);
	for(size_t i = 0; i < mSubMeshCount; ++i)
	{
		mSubMesh[i].Save(s);
	}
}

// 26-12-2005
void Mesh::exportToOBJ(char *nomfich)
{
	// Genera un fichero obj con vrtices, tringulos

	std::ofstream obj(nomfich);
	obj << "begin" << std::endl;
	// Vrtices
	for (size_t i=0; i<this->mSubMeshCount; i++) 
	{
		for (size_t j=0; j<this->mSubMesh[i].mVertexBuffer->mVertexCount; j++) 
		{	
			obj << "v " <<  this->mSubMesh[i].mVertexBuffer->mPosition[j].x << " " <<
				this->mSubMesh[i].mVertexBuffer->mPosition[j].y << " " <<
				this->mSubMesh[i].mVertexBuffer->mPosition[j].z << " " << std::endl;
		}
	}


	// Caras
	for (size_t i=0; i<this->mSubMeshCount; i++) 
	{
		for (size_t j=0; j<this->mSubMesh[i].mIndexCount; j=j+3) 
		{	
			obj << "f " <<  this->mSubMesh[i].mIndex[j]+1 << " " << 
				this->mSubMesh[i].mIndex[j+1]+1 << " " << 
				this->mSubMesh[i].mIndex[j+2]+1 << std::endl;
		}
	}

	obj << "end" << std::endl;
	obj.close();
}

// 26-12-2005
Geometry::Mesh *Mesh::toSharedVertex() 
{

	// Move all vertex to the shared vertex buffer.
	Geometry::Mesh *mesh = new Geometry::Mesh();

	if (this->mSubMesh[0].mSharedVertexBuffer)
	{
		*mesh	=	*this;
		return	mesh;
	}
	
	mesh->mVertexBuffer=new VertexBuffer();
	mesh->mType=this->mType;
	mesh->mSubMeshCount = this->mSubMeshCount;
	// Reserva memoria para los submeshes
	mesh->mSubMesh = new Geometry::SubMesh[this->mSubMeshCount];

	// construccin de los submeshes
	long int acumVerts=0;
	for (size_t i=0; i<mesh->mSubMeshCount; i++) 
	{
		mesh->mSubMesh[i].mSharedVertexBuffer=true;
		mesh->mSubMesh[i].mVertexBuffer=mesh->mVertexBuffer;
		mesh->mSubMesh[i].mStripCount=0;
		mesh->mSubMesh[i].mStrip=NULL;

		// copiar los ndices
		mesh->mSubMesh[i].mIndexCount=this->mSubMesh[i].mIndexCount;
		mesh->mSubMesh[i].mIndex=new Geometry::Index[this->mSubMesh[i].mIndexCount];

		for (size_t j=0; j< this->mSubMesh[i].mIndexCount; j++) 
		{ 
			mesh->mSubMesh[i].mIndex[j]	=	this->mSubMesh[i].mIndex[j]+acumVerts;
		}

		acumVerts	+=	this->mSubMesh[i].mVertexBuffer->mVertexCount;

		// Copiar las tiras
		int Desp=0; // desplazamiento entre posiciones de memoria.
		mesh->mSubMesh[i].mStripCount=this->mSubMesh[i].mStripCount;
		if (this->mSubMesh[i].mStripCount>0) 
		{
			mesh->mSubMesh[i].mStrip = new Index*[this->mSubMesh[i].mStripCount];

			Desp= &(mesh->mSubMesh[i].mIndex[0]) - &(this->mSubMesh[i].mIndex[0]);
			for (size_t j=0;j<this->mSubMesh[i].mStripCount;j++) 
			{
				mesh->mSubMesh[i].mStrip[j]=this->mSubMesh[i].mStrip[j] + Desp;
			}
		}
	}
	
	mesh->mVertexBuffer->mVertexCount=acumVerts;
	mesh->mVertexBuffer->mVertexInfo=this->mSubMesh[0].mVertexBuffer->mVertexInfo;
	mesh->mVertexBuffer->mPosition=new Geometry::Vector3[mesh->mVertexBuffer->mVertexCount];
	mesh->mVertexBuffer->mNormal=new Geometry::Vector3[mesh->mVertexBuffer->mVertexCount];
	mesh->mVertexBuffer->mTexCoords=new Geometry::Vector2[mesh->mVertexBuffer->mVertexCount];
	
	// copiar los vrtices
	acumVerts=0;
	size_t newIndex;
	for (size_t i=0; i<this->mSubMeshCount; i++) 
	{
		for(size_t j=0;j<this->mSubMesh[i].mVertexBuffer->mVertexCount;j++)
		{
			newIndex=acumVerts+j;
			mesh->mVertexBuffer->mPosition[newIndex].x=this->mSubMesh[i].mVertexBuffer->mPosition[j].x;
			mesh->mVertexBuffer->mPosition[newIndex].y=this->mSubMesh[i].mVertexBuffer->mPosition[j].y;
			mesh->mVertexBuffer->mPosition[newIndex].z=this->mSubMesh[i].mVertexBuffer->mPosition[j].z;

			mesh->mVertexBuffer->mNormal[newIndex].x=this->mSubMesh[i].mVertexBuffer->mNormal[j].x;
			mesh->mVertexBuffer->mNormal[newIndex].y=this->mSubMesh[i].mVertexBuffer->mNormal[j].y;
			mesh->mVertexBuffer->mNormal[newIndex].z=this->mSubMesh[i].mVertexBuffer->mNormal[j].z;

			mesh->mVertexBuffer->mTexCoords[newIndex].x=this->mSubMesh[i].mVertexBuffer->mTexCoords[j].x;
			mesh->mVertexBuffer->mTexCoords[newIndex].y=this->mSubMesh[i].mVertexBuffer->mTexCoords[j].y;
		}
		acumVerts+=this->mSubMesh[i].mVertexBuffer->mVertexCount;
	}

	return mesh;
}

