/*	==========================================================================
 *	(C) 2005 Universitat Jaume I
 *	==========================================================================
 *	PROYECT:	GAME TOOLS
 *	==========================================================================*/
/**	CONTENT:	
	*
	*
	*	@file	GeoMeshLoader.cpp
/*===========================================================================*/
#include	"GeoMeshLoader.h"

using namespace Geometry;
using	namespace	std;

//	Jump a chunk.
void GeoMeshLoader::jumpChunk(FILE	*f)
{
	fseek(f,-(long)CHUNK_OVERHEAD_SIZE,1);

	//	Return to the begin of the chunk.
	fseek(f,long_actual,SEEK_CUR);
}

//	Read a chunk.
unsigned short GeoMeshLoader::readChunk(FILE	*f)
{
	unsigned	short	id;

	fread(&id,sizeof(unsigned short),1,f);
	fread(&long_actual,sizeof(unsigned long),1,f);
									
	return id;
}

//	Read geometry vertex element.
void GeoMeshLoader::readGeometryVertexElement(FILE	*f, Mesh	*geoMesh)
{
	unsigned	short		source;
	GeometryElement		aux;
	int								i;
	bool							encontrado;
	VertexElementType	vType;

	i						=	0;
	encontrado	=	false;

	//		VertexElementSemantic vSemantic;
	// unsigned short source;  	// buffer bind source
	fread(&source,sizeof(unsigned short),1,f);

	// unsigned short type;    	// VertexElementType
	fread(&aux.type,sizeof(unsigned short),1,f);

	vType	=	static_cast<VertexElementType>(aux.type);

	if (vType == VET_FLOAT3)
	{
		printf("%u es float3 ",aux.type);
	}

	if (vType == VET_FLOAT4)
	{
		printf("%u es float4 ",aux.type);
	}

	if (vType == VET_FLOAT2)
	{
		printf("%u es float2 ",aux.type);
	}

	// unsigned short semantic; // VertexElementSemantic
	fread(&aux.semantic,sizeof(unsigned short),1,f);

	//	vSemantic = static_cast<VertexElementSemantic>(tmp);
	//	unsigned short offset;	// start offset in buffer in bytes
	fread(&aux.offset,sizeof(unsigned short),1,f);

	// unsigned short index;	// index of the semantic
	/*
	VES_POSITION  Position, 3 reals per vertex.  
	VES_BLEND_WEIGHTS  Blending weights.  
	VES_BLEND_INDICES  Blending indices.  
	VES_NORMAL  Normal, 3 reals per vertex.  
	VES_DIFFUSE  Diffuse colours.  
	VES_SPECULAR  Specular colours.  
	VES_TEXTURE_COORDINATES  Texture coordinates.  
	VES_BINORMAL  Binormal (Y axis if normal is Z).  
	VES_TANGENT  Tangent (X axis if normal is Z).  
	*/
	fread(&aux.index,sizeof(unsigned short),1,f);

	while ((i < list.size()) && (!encontrado))
	{
		if (list[i].source==source)
		{
			encontrado	=	true;
		}
		else
		{
			i++;
		}
	}

	if (encontrado)
	{
		list[i].list.push_back(aux);
	}
	else
	{
		GT	aux2;
		
		aux2.source	=	source;
		aux2.list.push_back(aux);
		list.push_back(aux2);
	}
	
	//	Debug.
	cout	<<	"Finish Vertex Element"	<<	endl;
}

//	Read geometry vertex declaration.
void GeoMeshLoader::readGeometryVertexDeclaration(FILE	*f, Mesh	*geoMesh)
{
	unsigned short chunkID;
  // Find optional geometry chunks
  if (!feof(f))
  {
    chunkID	=	readChunk(f);

    while(!feof(f) && (chunkID == M_GEOMETRY_VERTEX_ELEMENT ))
    {
      switch (chunkID)
      {
				case M_GEOMETRY_VERTEX_ELEMENT:
					cout	<<	"M_GEOMETRY_VERTEX_ELEMENT"	<<	endl;
          readGeometryVertexElement(f, geoMesh);
          break;
      }

      // Get next chunk
      if (!feof(f))
      {
				chunkID = readChunk(f);
      }
    }//	End while.

    if (!feof(f))
    {
			// Backpedal back to start of non-submesh chunk
			fseek(f,-(long)CHUNK_OVERHEAD_SIZE,1);
    }

  }//	End if (!feof(f))
}

//	Read geometry vertex buffer.
void GeoMeshLoader::readGeometryVertexBuffer(FILE	*f, Mesh	*geoMesh, int option)
{
	unsigned	short	bindIndex;
	unsigned	short	vertexSize;
	unsigned	short	headerID;
	int							i;
	float						aux;
	int							k;
	bool						encontrado;
	float						x,y,z;
	SubMesh					*geoSubMesh;
	VertexBuffer		*vertex_buffer;

	i						=	0;
	k						=	0;
	encontrado	=	false;

	
	if (option == GEOMESH_BUILD)
	{
		//	If is the main mesh.
		if (currentSubMesh == -1)
		{
			vertex_buffer	=	geoMesh->mVertexBuffer;
		}
		//	If is a submesh.
		else
		{
			//	Gets the current submesh
			geoSubMesh		=	&geoMesh->mSubMesh[currentSubMesh];
			vertex_buffer	=	geoSubMesh->mVertexBuffer;
		}
	}
	
	// Index to bind this buffer to
	fread(&bindIndex,sizeof(unsigned short),1,f);

	// Per-vertex size, must agree with declaration at this index
	fread(&vertexSize,sizeof(unsigned short),1,f);

	//	Debug.
	cout	<<	"Per-vertex size: "
				<<	vertexSize
				<<	endl;
	
	// Check for vertex data header
	headerID	=	readChunk(f);

	if (headerID != M_GEOMETRY_VERTEX_BUFFER_DATA)
	{
		cout	<<	"Can't find vertex buffer data area"
					<<	endl;
	}
	else
	{
		cout	<<	"M_GEOMETRY_VERTEX_BUFFER_DATA"	<<	endl;
	}

	while ((i<list.size())&&(!encontrado))
	{
		if (list[i].source	==	bindIndex)
		{
			encontrado	=	true;
		}
		else
		{
			i++;
		}
	}

	//	Considero q los van por grupitos de vertices+normales+...
	//	para cada uno de los vertices
	if (encontrado)
	{
		FILE * salida;
		salida=fopen(nom_salida,"a");
		cout	<<	"----->Leo los vertices"
					<<	endl;

		for (k=0;k<numVertices;k++)
		{
			//Para cada uno de los elementos de ese source.
			for(int j=0;j<list[i].list.size();j++)
			{
				//	Son posiciones, vertices
				if (list[i].list[j].semantic==1)
				{
					fread(&x,sizeof(float),1,f);
					fread(&y,sizeof(float),1,f);
					fread(&z,sizeof(float),1,f);

					if (option == GEOMESH_BUILD)
					{
						vertex_buffer->mVertexInfo		|=	VERTEX_POSITION;
						vertex_buffer->mPosition[k].x	=		x;
						vertex_buffer->mPosition[k].y	=		y;
						vertex_buffer->mPosition[k].z	=		z;
					}
				}
				//	si no son vertices no lo quiero
				else 
				{
					//Se tendra que hacer con un fseek
					if (list[i].list[j].type==2) //Son 3 floats
					{
						fread(&x,sizeof(float),1,f);
						fread(&y,sizeof(float),1,f);
						fread(&z,sizeof(float),1,f);

						if (option == GEOMESH_BUILD)
						{
							vertex_buffer->mVertexInfo	|=	VERTEX_NORMAL;
							vertex_buffer->mNormal[k].x	=		x;
							vertex_buffer->mNormal[k].y	=		y;
							vertex_buffer->mNormal[k].z	=		z;
						}
					}
					if (list[i].list[j].type==1) //Son 2 floats
					{
						fread(&x,sizeof(float),1,f);
						fread(&y,sizeof(float),1,f);
						
						if (option == GEOMESH_BUILD)
						{
							vertex_buffer->mVertexInfo			|=	VERTEX_TEXCOORDS;
							vertex_buffer->mTexCoords[k].x	=		x;
							vertex_buffer->mTexCoords[k].y	=		y;
						}
					}
				}
			}
		}
		fclose(salida);
	}
	else
	{
		cout	<<	"Error"
					<<	endl;
	}

}

//	Read geometry.
void GeoMeshLoader::readGeometry(FILE	*f, Mesh	*geoMesh, int option)
{
	SubMesh				*geoSubMesh;
	VertexBuffer	*vertex_buffer;
	
	fread(&numVertices,sizeof(unsigned int),1,f);

	if (option == GEOMESH_BUILD)
	{
		if (currentSubMesh == -1)
		{
			//	Get the current vertex buffer.
			vertex_buffer	=	geoMesh->mVertexBuffer;
		}
		else
		{
			//	Get the current submesh.
			geoSubMesh		=	&geoMesh->mSubMesh[currentSubMesh];

			//	Create the VertexBuffer of the submesh.
			geoSubMesh->mVertexBuffer	=	new VertexBuffer();

			//	Get the current vertex buffer.
			vertex_buffer	=	geoSubMesh->mVertexBuffer;
		}

		//	Initialize the VertexBuffer of the current submesh.
		vertex_buffer->mVertexCount	=	numVertices;

		//	Initialize the position array.
		vertex_buffer->mPosition	=	new Vector3[numVertices];

		//	Initialize the normal array.
		vertex_buffer->mNormal		=	new Vector3[numVertices];

		//	Initialize the texture coords array.
		vertex_buffer->mTexCoords	=	new Vector2[numVertices];
	}

	// Find optional geometry chunks
	if (!feof(f))
	{
		unsigned short chunkID = readChunk(f);
	
		while(!feof(f) && 
					(chunkID == M_GEOMETRY_VERTEX_DECLARATION || 
					chunkID == M_GEOMETRY_VERTEX_BUFFER ))
		{
			switch (chunkID)
			{
				case M_GEOMETRY_VERTEX_DECLARATION:
					cout<<"M_GEOMETRY_VERTEX_DECLARATION\n";
					readGeometryVertexDeclaration(f, geoMesh);
					break;
				case M_GEOMETRY_VERTEX_BUFFER:
					cout<<"M_GEOMETRY_VERTEX_BUFFER\n";
					readGeometryVertexBuffer(f, geoMesh,option);
					break;
			}
		
			// Get next chunk.
			if (!feof(f))
			{
				chunkID = readChunk(f);
			}
		}
		if (!feof(f))
		{
			// Backpedal back to start of non-submesh chunk.
			fseek(f,-(long)CHUNK_OVERHEAD_SIZE,1);
		}
	}
}

//	Read a submesh operation.
void GeoMeshLoader::readSubMeshOperation(	FILE	*f,
																					Mesh	*geoMesh,
																					int 	option)
{
	unsigned	short	opType;
	fread(&opType,sizeof(unsigned short),1,f);
	printf("\tLa render operation es %d\n",opType);

	if (option == GEOMESH_BUILD)
	{
		if (opType == 5)
		{
			geoMesh->mSubMesh[currentSubMesh].mType	=	GEO_TRIANGLE_STRIPS;
		}
		else
		{
			geoMesh->mSubMesh[currentSubMesh].mType	=	GEO_TRIANGLE_LIST;
		}
	}
}

//	Read a submesh.
void GeoMeshLoader::readSubMesh(FILE	*f, Mesh	*geoMesh, int option)
{
	SubMesh					*geoSubMesh;
	unsigned	short	chunkID;
	char						materialName[255];
	bool						useSharedVertices;
	unsigned	int		numIndices;
	bool						idx32bit;
	FILE						*salida;

	switch	(option)
	{
		case	SUBMESH_COUNT:
			geoMesh->mSubMeshCount++;
			break;
		case	GEOMESH_BUILD:

			//	Debug.
			cout	<<	"Build GEOMESH"	<<	endl;

			//	Initialize the new subMesh.
			geoSubMesh	=	&geoMesh->mSubMesh[currentSubMesh];

			cout	<<	"Current SubMesh: "
						<<	currentSubMesh
						<<	endl;
	}
	
	//La vacio antes de leer el mesh
	list.clear(); 

	fgets(materialName,255,f);
	printf("\tmaterialName: %s",materialName);

	fread(&useSharedVertices,sizeof(bool),1,f);
	
	if (option	==	GEOMESH_BUILD)
	{
		//	Obtain the flag of vertex shared.
		geoSubMesh->mSharedVertexBuffer	=	useSharedVertices;
		
		if (useSharedVertices)
		{
			//	The VB of the submesh points to the VB of the mesh.
			geoSubMesh->mVertexBuffer	=	geoMesh->mVertexBuffer;
			
			cout	<<	"\tUse Shared Vertices"
						<<	endl;

		}
	}

	fread(&numIndices,sizeof(unsigned int),1,f);
	printf("\tnumIndices: %u\n",numIndices);
	printf("\tnumTriangles: %u\n",numIndices/3);
	fread(&idx32bit,sizeof(bool),1,f);
	
	if (option == GEOMESH_BUILD)
	{
			//	Sets the index count of the submesh.
			geoSubMesh->mIndexCount	=	numIndices;
	
			//	Create the index arrar for this submesh.
			geoSubMesh->mIndex	=	new Index[numIndices];
	}
	
	if (idx32bit)
	{
		printf("\tSon 32 bits por indice\n");
	}
	else
	{
		printf("\tSon 16 bits por indice\n");
	}

	salida	=	fopen(nom_salida,"a");

	fprintf(salida,"\n#----->Leo un subMesh:");
	
	if (useSharedVertices)
	{
		fprintf(salida," shared vertices");
	}
	else
	{
		fprintf(salida,
						" no shared vertices %d triangles",
						numIndices/3);
	}

	fprintf(salida,"\n# Indexes:");
	
	//Sumo uno a los vertices porque en el obj no puede hacer un vertice 0
	if (idx32bit)
	{
		unsigned int aux;

		for (int i = 0; i < numIndices; i++)
		{
			fread(&aux,sizeof(unsigned int),1,f);

			if (option == GEOMESH_BUILD)
			{
					geoSubMesh->mIndex[i]	=	aux;
			}
		}
	}
	else
	{
		unsigned short aux;

		for (int i = 0; i < numIndices; i++)
		{
			fread(&aux,sizeof(unsigned short),1,f);
			
			if (option == GEOMESH_BUILD)
			{
				geoSubMesh->mIndex[i]	=	aux;
			}
		}
	}

	fclose(salida);

  // M_GEOMETRY chunk 
	// (Optional: present only if useSharedVertices = false)
  if (!useSharedVertices)
	{
		chunkID = readChunk(f);
		
		if (chunkID != M_GEOMETRY)
		{
			printf("Missing geometry data in mesh file\n"); //Error
		}
		
		readGeometry(f, geoMesh, option);
	}

	// Find all bone assignments (if present) 
	if (!feof(f))
	{
		chunkID = readChunk(f);
		while(!feof(f) &&
					(chunkID == M_SUBMESH_BONE_ASSIGNMENT ||
					chunkID == M_SUBMESH_OPERATION))
		{
			switch(chunkID)
			{
				case M_SUBMESH_OPERATION:
								
					cout<<"M_SUBMESH_OPERATION\n";

					readSubMeshOperation(f, geoMesh, option);
					
					break;
				case M_SUBMESH_BONE_ASSIGNMENT:
					jumpChunk(f);
					break;
			}

			if (!feof(f))
			{
				chunkID	=	readChunk(f);
			}

		}
		if (!feof(f))
		{
			// Backpedal back to start of chunk
			fseek(f,-(long)CHUNK_OVERHEAD_SIZE,1);
		}
	}

}

//	Read a mesh lod information.
void GeoMeshLoader::readMeshLodInfo(FILE	*f, Mesh	*geoMesh)
{
	unsigned	short	chunkID;
	unsigned	short	i;
	unsigned	short	aux;
	bool						aux2;

  // unsigned short numLevels;
	fread(&aux,sizeof(unsigned short),1,f);
	// bool manual;
	// (true for manual alternate meshes, false for generated)
	fread(&aux2,sizeof(bool),1,f);

	// Loop from 1 rather than 0 (full detail index is not in file)
	for (i = 1; i < aux; ++i)
	{
		chunkID = readChunk(f);
		if (chunkID != M_MESH_LOD_USAGE)
		{
			printf("Missing M_MESH_LOD_USAGE chunk\n"); //Error
		}
		else printf("\tM_MESH_LOD_USAGE (%d bytes) ",long_actual);
		jumpChunk(f);

	}
}

//	Read a submesh name table.
void GeoMeshLoader::readSubMeshNameTable(FILE	*f, Mesh	*geoMesh)
{
		unsigned	short	chunkID;
		unsigned	short	subMeshIndex;
		char						string[255];

		if (!feof(f))
		{
			chunkID	=	readChunk(f);
			
			while	(!feof(f) && (chunkID == M_SUBMESH_NAME_TABLE_ELEMENT ))
			{
				printf("M_SUBMESH_NAME_TABLE_ELEMENT %d bytes\n",long_actual);
				
				// Read in the index of the submesh.
				fread(&subMeshIndex,sizeof(unsigned short),1,f);
				
				// Read in the String and map it to its index.
				fgets(string,255,f);
				printf("index %u name %s\n",subMeshIndex,string);

				// If we're not end of file get the next chunk ID
				if (!feof(f))
				{
					chunkID = readChunk(f);
				}
			}
			if (!feof(f))
			{
				// Backpedal back to start of chunk
				fseek(f,-(long)CHUNK_OVERHEAD_SIZE,1);
			}
		}
}

//	Read a mesh file.
void GeoMeshLoader::readMesh(FILE	*f, Mesh	*geoMesh, int option)
{
	unsigned	short	chunkID;
	bool						nada;

	//	Debug.
	cout	<<	"Read Mesh"
				<<	endl;
	
	//	Create the VertexBuffer of the mesh.
	geoMesh->mVertexBuffer	=	new	VertexBuffer();
	geoMesh->mVertexBuffer->mVertexCount	=	0;

	//La vacio antes de leer el mesh
	list.clear();

	// bool skeletallyAnimated
	fread(&nada,sizeof(bool),1,f);

  // Find all subchunks 
	if (!feof(f))
	{
		chunkID	=	readChunk(f);

		while(!feof(f) &&
				(chunkID == M_GEOMETRY ||
				 chunkID == M_SUBMESH ||
				 chunkID == M_MESH_SKELETON_LINK ||
				 chunkID == M_MESH_BONE_ASSIGNMENT ||
				 chunkID == M_MESH_LOD ||
				 chunkID == M_MESH_BOUNDS ||
				 chunkID == M_SUBMESH_NAME_TABLE ||
				 chunkID == M_EDGE_LISTS))
		{
			switch(chunkID)
			{
				case M_GEOMETRY:

					//	Debug.
					cout	<<	"Geometry"	<<	endl;

					readGeometry(f, geoMesh, option);
					break;

				case M_SUBMESH:

					cout	<<	"SubMesh"	<<	endl;

					if (option == GEOMESH_BUILD)
					{
						//	Increments the current submesh.
						currentSubMesh++;
					}

					readSubMesh(f, geoMesh, option);

					break;

				case M_MESH_SKELETON_LINK:

					cout	<<	"M_MESH_SKELETON_LINK  "
						<<	long_actual
						<<	" bytes\n";

					jumpChunk(f);
					break;

				case M_MESH_BONE_ASSIGNMENT:

					cout	<<	"M_MESH_BONE_ASSIGNMENT  "
						<<	long_actual
						<<	" bytes\n";

					break;

				case M_MESH_LOD:

					cout	<<	"M_MESH_LOD\n";
					readMeshLodInfo(f, geoMesh);
					break;

				case M_MESH_BOUNDS:

					cout	<<	"M_MESH_BOUNDS "
						<<	long_actual
						<<	" bytes\n";

					jumpChunk(f);
					break;

				case M_SUBMESH_NAME_TABLE:

					cout	<<	"M_SUBMESH_NAME_TABLE "
						<<	long_actual
						<<	" bytes\n";

					readSubMeshNameTable(f,	geoMesh);
					break;

				case M_EDGE_LISTS:

					cout	<<	"M_EDGE_LISTS "
						<<	long_actual
						<<	" bytes\n";

					jumpChunk(f);
					break;
			}

			if (!feof(f))
			{
				chunkID	=	readChunk(f);
			}

		}

		if (!feof(f))
		{
			// Backpedal back to start of chunk
			fseek(f,-(long)CHUNK_OVERHEAD_SIZE,1);
			printf("Malo\n");
		}
	}

}

/*
		Allocate memory to the array of strips
*/
SubMesh* GeoMeshLoader::BuildStripsGeoSubMesh(SubMesh* geoSubMesh)
{
	bool		head_found;
	bool		tail_found;
	int			strip_count;
	int			tailIndex;
	size_t	tail;


	head_found							=	false;
	tail_found							=	false;
	geoSubMesh->mStripCount	=	1;

	//	Foreachone of the indices.
	//	Obtains the number of triangle strips.
	for (int i = 0; i < (geoSubMesh->mIndexCount - 1); i++)
	{
		if (geoSubMesh->mIndex[i] == geoSubMesh->mIndex[i+1])
		{
			if (head_found)
			{
				tail_found	=	true;
			}
			else
			{
				head_found	=	true;
			}

			//	Jump one index.
			i++;
		}
		else
		{
			//	Increments the number of triangle strips.
			if (tail_found)
			{
				geoSubMesh->mStripCount++;
			}

			head_found	=	false;
			tail_found	=	false;
		}
	}

	geoSubMesh->mStrip		=	(Index**) malloc(	sizeof(Index*)
			*
			geoSubMesh->mStripCount);

	//	Number of strips. Initially there is one triangle strip.
	strip_count	=	1;

	head_found						=	false;
	tail_found						=	false;

	//	Initialize the fist triangle strip.
	geoSubMesh->mStrip[0]	=	&geoSubMesh->mIndex[0];

	//	Foreachone of the indices.
	//	Assigns the beginning of the triangle strips.
	for(int i = 0;i < (geoSubMesh->mIndexCount - 1); i++)
	{
		if(geoSubMesh->mIndex[i] == geoSubMesh->mIndex[i+1])
		{
			if(head_found)
			{
				if (!tail_found)
				{
					tailIndex		=	i;
				}

				tail_found	=	true;
			}
			else
			{
				head_found	=	true;
			}

			//	Jump one index.
			i++;
		}
		else
		{
			//	Increments the number of triangle strips.
			if(tail_found)
			{
				geoSubMesh->mStrip[strip_count++]	=	&geoSubMesh->mIndex[tailIndex];
			}

			head_found	=	false;
			tail_found	=	false;
		}
	}

	return	geoSubMesh;
}

//	Loads a mesh.
Mesh* GeoMeshLoader::load(char	*nameFileMesh) 
{
	unsigned	short	uno;
  unsigned	short	chunkID;
	char						version[255];
	char						kk;
	FILE						*salida;
	FILE						*pFile;
	SubMesh					*geosubmesh;

	//	Initialize the current submesh;
	currentSubMesh	=	-1;
	
	std::cout	<<	"Load a new Mesh"
						<<	std::endl;

	//	Initialize the return value.
	geoMesh	=	new	Mesh();

	//	Open the mesh file.
	pFile	=	fopen( nameFileMesh, "rb" );

  if (!pFile)
  {
		printf("No he encontrado el fichero\n");
		exit(0);
  }

	//	Count the submeshes
	//	and next build the geomesh.
	for (int option = 0; option < 2;option++)
	{
		fread(&uno,sizeof(unsigned short),1,pFile);

		if (uno != M_HEADER)
		{
			cout	<<	"no se ha encontrado el header"
						<<	endl;
		}

		// Read version.
		fgets(version,255,pFile);

		cout	<<	version	<<	endl;

		if (strcmp(version,"[MeshSerializer_v1.30]\n"))
		{
			cout	<<
						"No se puede trabajar con versiones de"
						<<
						" .mesh diferentes de la 1.3"
						<<
						endl;
		}

		salida	=	fopen(nom_salida,"w");

		fprintf(salida,"# Empiezo el fichero:");

		fclose(salida);

		while(!feof(pFile))
		{
			chunkID = readChunk(pFile);

			switch (chunkID)
			{
				case M_MESH:
					readMesh(pFile, geoMesh, option);
					break;
			}
		}

		//	Create the submesh array
		if (option == SUBMESH_COUNT)
		{		
			cout	<<	"---------------"
						<<	endl;

			cout	<<	"SubMesh Count: "
						<<	geoMesh->mSubMeshCount
						<<	endl;

			geoMesh->mSubMesh	= new SubMesh[geoMesh->mSubMeshCount];
		}
							
		if (feof(pFile))
		{
			cout	<<	"------->Hemos llegado al final del fichero"
						<<	endl;
		}

		//cout	<<	"Press enter to exit\n";

		//	Move the curso to the begining of the file.
		fseek(pFile,0,SEEK_SET);
	}

	//	Goes to the end of the file.
	fseek(pFile,0,SEEK_END);

	//	Gets the size of the file.
	mFileSize	=	ftell(pFile);
	
	// Close the mesh file.
	fclose(pFile);
	
	for (int submesh = 0;	submesh < geoMesh->mSubMeshCount; submesh++)
	{
		//	Gets the actual submesh.
		geosubmesh	=	&geoMesh->mSubMesh[submesh];
		
		if (geosubmesh->mType == Geometry::GEO_TRIANGLE_STRIPS)
		{
			//	Fill the strips list.
			geosubmesh	=	BuildStripsGeoSubMesh(geosubmesh);
		}
	}
	
	return	geoMesh;
}

//	Get the size in bytes of the file.
size_t	GeoMeshLoader::getFileSize()
{
	return	mFileSize;
}

//	Constructor
GeoMeshLoader::GeoMeshLoader()
{
	nom_salida	=	"salida22.obj";
	geoMesh			=	NULL;
	mFileSize		=	0;
}

//	Destroyer.
GeoMeshLoader::~GeoMeshLoader()
{
	delete	geoMesh;
}

